-- The counter block of IT
--
-- Signals:
--   cnt_in: cnt[i], counter carry signal
--   cnt_out: cnt[i+1], counter carry signal
--   count: count[i], count signal
--
-- This is a 4-bit counter. In this VHDL mode of CCM, there are 15 ITs
-- in the ILU, which means the counter should be able to count from 0 
-- to 15. As described in the thesis, 4-bit counter can count from 0 to
-- 14. By combine the "count" signal of the last IT, we are able to 
-- count from 0 to 15. By using 4-bit counter instead of 5-bit counter,
-- we save 1 iterative signal. The decoder of this counter is described 
-- by pcountd.vhd

library ieee;
use ieee.std_logic_1164.all;
use work.all;
use work.parts.all;

entity it_count is
  port (cnt_in  : in std_logic_vector (3 downto 0);
        count   : in std_logic;
        cnt_out : out std_logic_vector (3 downto 0)
       );
end;

architecture arch of it_count is

  signal tmp_sig : std_logic_vector(1 to 1);

begin

  u0: parts.mux21N generic map (size=>1) port map
   ( din0=>cnt_in(0 downto 0), din1=>cnt_in(1 downto 1), 
     sel=>count, dout=>cnt_out(0 downto 0));

  u1: parts.mux21N generic map (1) port map
   ( din0=>cnt_in(1 downto 1), din1=>cnt_in(2 downto 2), 
     sel=>count, dout=>cnt_out(1 downto 1));

  u2: parts.mux21N generic map (1) port map
   ( din0=>cnt_in(2 downto 2), din1=>cnt_in(3 downto 3), 
     sel=>count, dout=>cnt_out(2 downto 2));

  tmp_sig(1) <= cnt_in(0) xor cnt_in(1);

  u3: parts.mux21N generic map (1) port map
   ( din0=>cnt_in(3 downto 3), din1=>tmp_sig,
     sel=>count, dout=>cnt_out(3 downto 3));

end arch;

