/* File: cubeoper.c */

#include <stdio.h>
#include "cubeoper.h"

extern int debug;

/* get a literal of the given cube */
void getvar(CUBE *pcube, short idx, short *pvalue)
\{ CUBE temp;

  temp = *pcube;
  *pvalue = (temp >> (30-idx*2)) & 0x03L;
\}

/* set a literal of the given cube */
void setvar(CUBE *pcube, short idx, short *pvalue)
\{ CUBE temp;

  temp = *pvalue & 0x0003;
  *pcube = (*pcube) & (~(0x03L << (30-idx*2)));
  *pcube = *pcube | (temp << (30-idx*2));
\}

/* store a cube into the given array of cubes */
void storecube(ARRAYOFCUBES *parray, CUBE *pcube)
\{ 
  parray->cube[parray->numcube] = *pcube;
  parray->numcube += 1;
\}

/* skip the rest of line when read file */
void skip_line(FILE *fp)
\{ int ch;

  while ((ch=getc(fp)) != EOF && ch != '\verb!\!n');
\}

/* get a word from the file, the words are seperated by white spaces */
char *get_word(FILE *fp, char *word)
\{ int ch, i = 0;

  while ((ch = getc(fp)) != EOF && isspace(ch));

  word[i++] = ch;
  while ((ch = getc(fp)) != EOF && ! isspace(ch))
   \{ word[i++] = ch;
   \}
  word[i++] = '\verb!\!0';

  return word;
\}

/* convert a cube from PLA format to positional notation format */
int str2pn(char *str, int nvar, CUBE *pcube)
\{ short i, val, res=0;
  
  *pcube = 0L;

  if ( (strlen(str) != nvar) || (nvar >= 16))
   \{ printf("wrong length (%s,%d).\verb!\!n", str, nvar);
     res = 1;
   \}
  else
   \{ for (i=0; i<nvar; i++)
      \{ switch (str[i])
         \{ case '0': val=2; break;
           case '1': val=1; break;
           case '-': val=3; break;
           default : res = 2;
         \}
        setvar(pcube, i, &val);
      \}
   \}

  return res;
\}

/* convert a cube from positional notation format to PLA format */
void pn2str(CUBE *pcube, int nvar, char *str)
\{ short i, val;
  
  for (i=0; i<nvar; i++)
   \{ getvar(pcube, i, &val);

     switch (val)
      \{ case 0: str[i] = 'e'; break;
        case 1: str[i] = '1'; break;
        case 2: str[i] = '0'; break;
        case 3: str[i] = '-'; 
      \}
   \}
  str[nvar] = '\verb!\!0';
\}

void read_cubes(FILE *fp, ARRAYOFCUBES *cubes)
\{
  int i, ch, lineno, numvar, res;
  char word[256];
  CUBE tempcube;

  while (1)
   \{ switch(ch = getc(fp)) 
      \{ case EOF: return;
    
        case '\verb!\!n': 
        case ' ': 
        case '\verb!\!t': break;
    
        case '#': (void) ungetc(ch, fp);
                  skip_line(fp);
                  break;
    
        case '.':
          get_word(fp, word);

          /* .i gives the cube input size (binary-functions only) */
          if ( strcmp(word, "i") == 0 ) 
            \{ if (fscanf(fp, "%d", &numvar) != 1) 
                printf("error reading .i");
    
              cubes->numvar = numvar;
              /* printf("there are %d variables\verb!\!n", numvar); */
            \}
    
           /* .e and .end specify the end of the file */
           if (strcmp(word, "e") == 0) return;
    
           break;
    
        default:
           (void) ungetc(ch, fp);
           get_word(fp, word);
           /* printf("cube is >%s< ", word); */
           res = str2pn(word, numvar, &tempcube);
           /* printf("res=%d, cube is >%08X< \verb!\!n", res, tempcube); */
           storecube(cubes, &tempcube);
       \}
   \}
\}

/* display an array of cubes */
void showarray(ARRAYOFCUBES *parray, char *name)
\{ int  i;
  char word[17];

  printf("Array Of Cubes [%s]: %d variables, %d cubes\verb!\!n",
          name, parray->numvar, parray->numcube);

  for (i=0; i<parray->numcube; i++)
   \{ printf(" cube[%d]: %08X (", i, parray->cube[i]);
     pn2str(&parray->cube[i], parray->numvar, word);
     printf("%s)\verb!\!n", word);
   \}
\}         

/* disjoint sharp operation on two cubes */
void dsharp(CUBE cubea, CUBE cubeb, int numvar, ARRAYOFCUBES *parray)
\{ short vala, valb, valc;
  int i, j, res=0;
  CUBE tempcube = 0L;

  int subset[4][4] = \{ 1, 1, 1, 1,
                       0, 1, 0, 1,
                       0, 0, 1, 1,
                       0, 0, 0, 1 \};
 
  int active[4][4] = \{ 0, 0, 0, 0,
                       1, 0, 1, 0,
                       2, 2, 0, 0,
                       3, 2, 1, 0 \};

  /* check the first pre-relation */
  for (i=0; i<numvar; i++)
   \{ getvar(&cubea, i, &vala);
     getvar(&cubeb, i, &valb);

     if (!(vala & valb)) /* there is no intersection */ 
      \{ res = 1;
        break;
      \}
   \}
  if (res == 1) 
   \{ storecube(parray, &cubea);
     return;
   \}

  /* check the second pre-relation */
  for (i=0; i<numvar; i++)
   \{ getvar(&cubea, i, &vala);
     getvar(&cubeb, i, &valb);

     if (!subset[vala][valb]) /* A is not the subset of B */
      \{ res = 1;
        break;
      \}
   \}
  if (res != 1) return;  /* no result cube */

  /* calculate disjoint sharp */
  for (i=0; i<numvar; i++)
   \{ getvar(&cubea, i, &vala);
     getvar(&cubeb, i, &valb);

     if (!subset[vala][valb]) /* i is the special position */
      \{ tempcube = 0L;

        /* printf("the position %d is a special positon ...\verb!\!n", i); */

        for (j=0; j<i; j++)  /* after function */
         \{ getvar(&cubea, j, &vala);
           getvar(&cubeb, j, &valb);
           valc = vala & valb;
           setvar(&tempcube, j, &valc);
         \}

        getvar(&cubea, i, &vala);
        getvar(&cubeb, i, &valb);
        valc = active[vala][valb];
        setvar(&tempcube, i, &valc);

        for (j=i+1; j<numvar; j++)  /* before function */
         \{ getvar(&cubea, j, &vala);
           valc = vala;
           setvar(&tempcube, j, &valc);
         \}

        storecube(parray, &tempcube);
      \}
   \}
\}

/* copy one array of cubes to another array of cubes */
void copyarrcube(ARRAYOFCUBES *parraya, ARRAYOFCUBES *parrayb)
\{ int i;

  parrayb->numvar = parraya->numvar;
  parrayb->numcube = parraya->numcube;
  
  for (i=0; i<parraya->numcube; i++)
    parrayb->cube[i] = parraya->cube[i];
\}

/* disjoint sharp operation on two arrays of cubes */
void dsharparr(ARRAYOFCUBES *parraya,ARRAYOFCUBES *parrayb,ARRAYOFCUBES *pres)
\{ ARRAYOFCUBES tmparraya, tmparrayb;
  ARRAYOFCUBES *pcubesa, *pcubesb, *ptmp;
  int i, j;

  if (parraya->numvar != parrayb->numvar)
   \{ pres->numvar = 0;
     pres->numcube = 0;
     return;
   \}

  copyarrcube(parraya, &tmparraya);
  pcubesa = &tmparraya;
  pcubesb = &tmparrayb;

  pcubesb->numvar = parraya->numvar;

  /* showarray(pcubesa, "Copy arrA to tmp buf A"); */

  for (i=0; i<parrayb->numcube; i++)
   \{ pcubesb->numcube = 0;
     for (j=0; j<pcubesa->numcube; j++)
       dsharp(pcubesa->cube[j], parrayb->cube[i], pcubesa->numvar, pcubesb);

     if (debug) printf("Iteration %d: %d cubes\verb!\!n", i, pcubesb->numcube);
/*   showarray(pcubesb, "intermedia result"); 
*/
     ptmp = pcubesa;
     pcubesa = pcubesb;
     pcubesb = ptmp;
   \}
  
  /* showarray(pcubesa, "final result"); */
  copyarrcube(pcubesa, pres);
  pres->numvar = pcubesa->numvar;
  pres->numcube = pcubesa->numcube;
\}

/* calculate the difference time between two moments (two timeval structure) */
void difftv(struct timeval *tv1, struct timeval *tv2, struct timeval *tvdiff)
\{
  if (tv2->tv_usec < tv1->tv_usec)
   \{ tvdiff->tv_usec = tv2->tv_usec - tv1->tv_usec + 1000000;
     tvdiff->tv_sec = tv2->tv_sec - tv1->tv_sec - 1;
   \}
  else
   \{ tvdiff->tv_usec = tv2->tv_usec - tv1->tv_usec;
     tvdiff->tv_sec = tv2->tv_sec - tv1->tv_sec;
   \}
\}

